@@ -143,6 +143,10 @@ public:
 
 	// info
 	bool info(unsigned inf) const { return bp->info(inf); }
+	bool is_integer() const { return bp->is_integer(); }
+	bool is_real() const { return bp->is_real(); }
+	bool is_positive() const { return bp->is_positive(); }
+	bool is_negative() const { return bp->is_negative(); }
 
 	// operand access
 	size_t nops() const { return bp->nops(); }
@@ -150,6 +154,7 @@ public:
         bool get_first_symbol(ex &x) const;
         symbolset symbols() const;
         symbolset free_symbols() const;
+        std::unordered_set<unsigned> functions() const;
 	const ex op(size_t i) const { return bp->op(i); }
 	ex sorted_op(size_t i) const;
 	ex operator[](const ex & index) const { return (*bp)[index]; }
@@ -157,6 +162,7 @@ public:
 	ex & let_op(size_t i);
 	ex & operator[](const ex & index);
 	ex & operator[](size_t i);
+        void set_epseq_from(size_t i, ex e) { bp->set_epseq_from(i, e); }
 	ex lhs() const;
 	ex rhs() const;
 
@@ -172,7 +178,8 @@ public:
 	bool match(const ex & pattern, lst & repl_lst) const { return bp->match(pattern, repl_lst); }
 
 	// substitutions
-	ex subs(const exmap & m, unsigned options = 0) const;
+	ex subs(const exmap & m, unsigned options = 0) const
+                { return bp->subs(m, options); }
 	ex subs(const lst & ls, const lst & lr, unsigned options = 0) const;
 	ex subs(const ex & e, unsigned options = 0) const;
 
@@ -188,16 +195,17 @@ public:
 
 	// degree/coeff
 	bool is_polynomial(const ex & vars) const;
-	int degree(const ex & s) const { return bp->degree(s); }
-	int ldegree(const ex & s) const { return bp->ldegree(s); }
+	numeric degree(const ex & s) const;
+	numeric ldegree(const ex & s) const;
 	ex coeff(const ex & s, const ex & n) const { return bp->coeff(s, n); }
-	ex lcoeff(const ex & s) const { return coeff(s, degree(s)); }
-	ex tcoeff(const ex & s) const { return coeff(s, ldegree(s)); }
+	ex lcoeff(const ex & s) const;
+	ex tcoeff(const ex & s) const;
         void coefficients(const ex & s, expairvec & vec) const;
 
 	// expand/collect
 	ex expand(unsigned options=0) const;
 	ex collect(const ex & s, bool distributed = false) const { return bp->collect(s, distributed); }
+        ex collect_powers() const;
 
 	// differentiation and series expansion
 	ex diff(const symbol & s, unsigned nth = 1) const;
@@ -233,6 +241,7 @@ public:
 	numeric max_coefficient() const;
         bool is_linear(const symbol& x, ex& a, ex& b) const;
         bool is_quadratic(const symbol& x, ex& a, ex& b, ex& c) const;
+        bool is_binomial(const symbol& x, ex& a, ex& j, ex& b, ex& n) const;
 
 	// domains
 	void set_domain(unsigned d);
@@ -241,17 +250,9 @@ public:
 	int compare(const ex & other) const;
 	bool is_equal(const ex & other) const;
 	bool is_zero() const;
-        bool is_integer_one() const;
-        bool is_integer_pmone() const;
+        bool is_one() const;
+        bool is_minus_one() const;
 	
-	// symmetry
-	ex symmetrize() const;
-	ex symmetrize(const lst & l) const;
-	ex antisymmetrize() const;
-	ex antisymmetrize(const lst & l) const;
-	ex symmetrize_cyclic() const;
-	ex symmetrize_cyclic(const lst & l) const;
-
 	// noncommutativity
 	unsigned return_type() const { return bp->return_type(); }
 	tinfo_t return_type_tinfo() const { return bp->return_type_tinfo(); }
@@ -644,9 +645,6 @@ inline const_postorder_iterator ex::postorder_end() const throw()
 	return const_postorder_iterator();
 }
 
-
-// utility functions
-
 /** Compare two objects of class quickly without doing a deep tree traversal.
  *  @return "true" if they are equal
  *          "false" if equality cannot be established quickly (e1 and e2 may
@@ -656,6 +654,12 @@ inline bool are_ex_trivially_equal(const ex &e1, const ex &e2)
 	return e1.bp == e2.bp;
 }
 
+
+// Make it possible to print exvectors and exmaps
+std::ostream & operator<<(std::ostream & os, const exvector & e);
+std::ostream & operator<<(std::ostream & os, const exset & e);
+std::ostream & operator<<(std::ostream & os, const exmap & e);
+
 /* Function objects for STL sort() etc. */
 struct ex_is_less : public std::binary_function<ex, ex, bool> {
 	bool operator() (const ex &lh, const ex &rh) const { return lh.compare(rh) < 0; }
@@ -673,137 +677,6 @@ struct ex_swap : public std::binary_function<ex, ex, void> {
 	void operator() (ex &lh, ex &rh) const { lh.swap(rh); }
 };
 
-
-// Make it possible to print exvectors and exmaps
-std::ostream & operator<<(std::ostream & os, const exvector & e);
-std::ostream & operator<<(std::ostream & os, const exset & e);
-std::ostream & operator<<(std::ostream & os, const exmap & e);
-
-// wrapper functions around member functions
-inline size_t nops(const ex & thisex)
-{ return thisex.nops(); }
-
-inline ex expand(const ex & thisex, unsigned options = 0)
-{ return thisex.expand(options); }
-
-inline ex conjugate(const ex & thisex)
-{ return thisex.conjugate(); }
-
-inline ex real_part(const ex & thisex)
-{ return thisex.real_part(); }
-
-inline ex imag_part(const ex & thisex)
-{ return thisex.imag_part(); }
-
-inline bool has(const ex & thisex, const ex & pattern, unsigned options = 0)
-{ return thisex.has(pattern, options); }
-
-inline bool find(const ex & thisex, const ex & pattern, lst & found)
-{ return thisex.find(pattern, found); }
-
-inline bool is_polynomial(const ex & thisex, const ex & vars)
-{ return thisex.is_polynomial(vars); }
-
-inline int degree(const ex & thisex, const ex & s)
-{ return thisex.degree(s); }
-
-inline int ldegree(const ex & thisex, const ex & s)
-{ return thisex.ldegree(s); }
-
-inline ex coeff(const ex & thisex, const ex & s, int n=1)
-{ return thisex.coeff(s, n); }
-
-inline ex numer(const ex & thisex)
-{ return thisex.numer(); }
-
-inline ex denom(const ex & thisex)
-{ return thisex.denom(); }
-
-inline ex numer_denom(const ex & thisex)
-{ return thisex.numer_denom(); }
-
-inline ex normal(const ex & thisex, int level=0, bool noexpand_combined=false,
-                bool noexpand_numer=true)
-{ return thisex.normal(level, noexpand_combined, noexpand_numer); }
-
-inline ex to_rational(const ex & thisex, lst & repl_lst)
-{ return thisex.to_rational(repl_lst); }
-
-inline ex to_rational(const ex & thisex, exmap & repl)
-{ return thisex.to_rational(repl); }
-
-inline ex to_polynomial(const ex & thisex, exmap & repl)
-{ return thisex.to_polynomial(repl); }
-
-inline ex to_polynomial(const ex & thisex, lst & repl_lst)
-{ return thisex.to_polynomial(repl_lst); }
-
-inline ex collect(const ex & thisex, const ex & s, bool distributed = false)
-{ return thisex.collect(s, distributed); }
-
-inline ex eval(const ex & thisex, int level = 0)
-{ return thisex.eval(level); }
-
-inline ex evalf(const ex & thisex, int level = 0, PyObject* parent=nullptr)
-{ return thisex.evalf(level, parent); }
-
-inline ex diff(const ex & thisex, const symbol & s, unsigned nth = 1)
-{ return thisex.diff(s, nth); }
-
-inline ex series(const ex & thisex, const ex & r, int order, unsigned options = 0)
-{ return thisex.series(r, order, options); }
-
-inline bool match(const ex & thisex, const ex & pattern, lst & repl_lst)
-{ return thisex.match(pattern, repl_lst); }
-
-inline ex symmetrize(const ex & thisex)
-{ return thisex.symmetrize(); }
-
-inline ex symmetrize(const ex & thisex, const lst & l)
-{ return thisex.symmetrize(l); }
-
-inline ex antisymmetrize(const ex & thisex)
-{ return thisex.antisymmetrize(); }
-
-inline ex antisymmetrize(const ex & thisex, const lst & l)
-{ return thisex.antisymmetrize(l); }
-
-inline ex symmetrize_cyclic(const ex & thisex)
-{ return thisex.symmetrize_cyclic(); }
-
-inline ex symmetrize_cyclic(const ex & thisex, const lst & l)
-{ return thisex.symmetrize_cyclic(l); }
-
-inline ex op(const ex & thisex, size_t i)
-{ return thisex.op(i); }
-
-inline ex lhs(const ex & thisex)
-{ return thisex.lhs(); }
-
-inline ex rhs(const ex & thisex)
-{ return thisex.rhs(); }
-
-inline bool is_zero(const ex & thisex)
-{ return thisex.is_zero(); }
-
-inline void swap(ex & e1, ex & e2)
-{ e1.swap(e2); }
-
-inline ex ex::subs(const exmap & m, unsigned options) const
-{
-	return bp->subs(m, options);
-}
-
-inline ex subs(const ex & thisex, const exmap & m, unsigned options = 0)
-{ return thisex.subs(m, options); }
-
-inline ex subs(const ex & thisex, const lst & ls, const lst & lr, unsigned options = 0)
-{ return thisex.subs(ls, lr, options); }
-
-inline ex subs(const ex & thisex, const ex & e, unsigned options = 0)
-{ return thisex.subs(e, options); }
-
-
 /* Convert function pointer to function object suitable for map(). */
 class pointer_to_map_function : public map_function {
 protected:
@@ -931,33 +804,7 @@ inline const T &ex_to(const ex &e)
 	return static_cast<const T &>(*e.bp);
 }
 
+
 } // namespace GiNaC
 
-
-// Specializations of Standard Library algorithms
-namespace std {
-
-/** Specialization of std::swap() for ex objects. */
-template <>
-inline void swap(GiNaC::ex &a, GiNaC::ex &b)
-{
-	a.swap(b);
-}
-
-/** Specialization of std::iter_swap() for vector<ex> iterators. */
-template <>
-inline void iter_swap(vector<GiNaC::ex>::iterator i1, vector<GiNaC::ex>::iterator i2)
-{
-	i1->swap(*i2);
-}
-
-/** Specialization of std::iter_swap() for list<ex> iterators. */
-template <>
-inline void iter_swap(list<GiNaC::ex>::iterator i1, list<GiNaC::ex>::iterator i2)
-{
-	i1->swap(*i2);
-}
-
-} // namespace std
-
 #endif // ndef __GINAC_EX_H__
