@@ -69,7 +69,6 @@ unsigned NAME##_SERIAL::serial = \
 namespace GiNaC {
 
 class function;
-class symmetry;
 
 typedef ex (* eval_funcp)();
 typedef ex (* evalf_funcp)(PyObject* parent);
@@ -80,7 +79,7 @@ typedef ex (* derivative_funcp)();
 typedef ex (* expl_derivative_funcp)();
 typedef ex (* power_funcp)();
 typedef ex (* series_funcp)();
-typedef ex (* subs_funcp)(PyObject* parent);
+typedef ex (* subs_funcp)();
 typedef void (* print_funcp)();
 
 // the following lines have been generated for max. 14 parameters
@@ -117,6 +116,9 @@ typedef ex (* power_funcp_3)(const ex &, const ex &, const ex &, const ex &);
 typedef ex (* series_funcp_1)(const ex &, const relational &, int, unsigned);
 typedef ex (* series_funcp_2)(const ex &, const ex &, const relational &, int, unsigned);
 typedef ex (* series_funcp_3)(const ex &, const ex &, const ex &, const relational &, int, unsigned);
+typedef ex (* subs_funcp_1)(const exmap&, const ex &);
+typedef ex (* subs_funcp_2)(const exmap&, const ex &, const ex &);
+typedef ex (* subs_funcp_3)(const exmap&, const ex &, const ex &, const ex &);
 
 typedef void (* print_funcp_1)(const ex &, const print_context &);
 typedef void (* print_funcp_2)(const ex &, const ex &, const print_context &);
@@ -194,6 +196,10 @@ public:
     function_options & series_func(series_funcp_2 s);
     function_options & series_func(series_funcp_3 s);
 
+    function_options & subs_func(subs_funcp_1 s);
+    function_options & subs_func(subs_funcp_2 s);
+    function_options & subs_func(subs_funcp_3 s);
+
     template <class Ctx> function_options & print_func(print_funcp_1 p)
     {
     	test_and_set_nparams(1);
@@ -221,7 +227,7 @@ public:
 	function_options & derivative_func(derivative_funcp_exvector d);
 	function_options & power_func(power_funcp_exvector d);
 	function_options & series_func(series_funcp_exvector s);
-
+	function_options & subs_func(series_funcp_exvector s);
 	function_options & derivative_func(derivative_funcp_exvector_symbol d);
 
 	template <class Ctx> function_options & print_func(print_funcp_exvector p)
@@ -248,7 +254,6 @@ public:
 	function_options & remember(unsigned size, unsigned assoc_size=0,
 	                            unsigned strategy=remember_strategies::delete_never);
 	function_options & overloaded(unsigned o);
-	function_options & set_symmetry(const symmetry & s);
 
 	std::string get_name() const { return name; }
 	unsigned get_nparams() const { return nparams; }
@@ -281,6 +286,7 @@ public:
 
 	unsigned nparams;
 
+	eval_funcp pynac_eval_f;
 	eval_funcp eval_f;
 	evalf_funcp evalf_f;
 	conjugate_funcp conjugate_f;
@@ -354,6 +360,8 @@ public:
     function(unsigned ser, const ex & param1);
     function(unsigned ser, const ex & param1, const ex & param2);
     function(unsigned ser, const ex & param1, const ex & param2, const ex & param3);
+    function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4);
+    function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4, const ex & param5);
     function(unsigned ser, const ex & param1, const ex & param2, const ex & param3, const ex & param4, const ex & param5, const ex & param6);
 	// end of generated lines
 	function(unsigned ser, exprseq  es);
@@ -371,6 +379,7 @@ public:
 	ex series(const relational & r, int order, unsigned options = 0) const override;
         void useries(flint_series_t& fp, int order) const override;
         ex subs(const exmap & m, unsigned options = 0) const override;
+	ex normal(exmap & repl, exmap & rev_lookup, int level = 0, unsigned options = 0) const override;
 	ex thiscontainer(const exvector & v) const override;
 	ex thiscontainer(std::unique_ptr<exvector> vp) const override;
 	ex conjugate() const override;
@@ -427,6 +436,11 @@ inline bool is_the_function(const ex & x)
 #define is_ex_the_function(OBJ, FUNCNAME) (GiNaC::is_the_function<FUNCNAME##_SERIAL>(OBJ))
 
 bool has_function(const ex & x);
+bool has_function(const ex& x, const std::string& s);
+bool has_function(const std::vector<std::string>& v);
+bool has_function(const ex& x,
+                const std::vector<std::string>& v,
+                bool all);
 bool has_symbol_or_function(const ex & x);
 
 } // namespace GiNaC
